package onig4j.java.util.regex;

import org.junit.Test;
import static org.junit.Assert.*;

/**
 *
 * @author calico
 */
public class MatcherTest {

    @Test
    public void start() {
        final String regex = ",";
        final String input = "abc,ｄ,eｆg,h,ｉ,,j,,,,ｋ,,,";
        
        final java.util.regex.Matcher matcher = java.util.regex.Pattern.compile(regex).matcher(input);
        assertTrue(matcher.find());
        assertEquals(3, matcher.start());
        
        Pattern pattern = Pattern.compile(regex);
        final Matcher matcher2 = pattern.matcher(input);
        assertTrue(matcher2.find());
        assertEquals(matcher.start(), matcher2.start());

        assertEquals(matcher.groupCount(), matcher2.groupCount());
        assertEquals(matcher.group(), matcher2.group());
        
        for (int i = 0; i < matcher.groupCount(); ++i) {
            assertEquals(matcher.start(i), matcher2.start(2));
            assertEquals(matcher.end(i), matcher2.end(2));
            assertEquals(matcher.group(i), matcher2.group(i));
        }
        
        matcher2.reset();
        assertEquals(0, matcher2.groupCount());
    }

    @Test
    public void group() {
        final int[] options
                = new int[] {
                        java.util.regex.Pattern.MULTILINE | java.util.regex.Pattern.UNIX_LINES,            
                        java.util.regex.Pattern.MULTILINE,            
                    };
        final String regex = "[^$]";
        final String input = "\r\nあいうえお\nかきくけこ\r\nさしすせそ\rたちつてと";
        
        for (final int option : options) {
            java.util.regex.Matcher matcher = java.util.regex.Pattern.compile(regex, option).matcher(input);
            assertTrue(matcher.find());
            String group = matcher.group();
            assertEquals(0, matcher.start());
            assertEquals(1, matcher.end());

            Pattern pattern = Pattern.compile(regex, option);
            Matcher matcher2 = pattern.matcher(input);
            assertTrue(matcher2.find());
            assertEquals(matcher.start(), matcher2.start());

            assertEquals(matcher.groupCount(), matcher2.groupCount());
            assertEquals(group, matcher2.group());
            assertEquals(matcher.start(), matcher2.start());
            assertEquals(matcher.end(), matcher2.end());

            for (int i = 0; i < matcher.groupCount(); ++i) {
                assertEquals(matcher.start(i), matcher2.start(2));
                assertEquals(matcher.end(i), matcher2.end(2));
                assertEquals(matcher.group(i), matcher2.group(i));
            }
        }
    }

    @Test(expected = UnsupportedOperationException.class)
    public void CANON_EQ() {
        final String regex = "a\u030A";
        final String input = "あいうえお\u00E5かきくけこ";
        
        java.util.regex.Matcher matcher = java.util.regex.Pattern.compile(regex).matcher(input);
        assertFalse(matcher.find());
        
        matcher = java.util.regex.Pattern.compile(regex, java.util.regex.Pattern.CANON_EQ).matcher(input);
        assertTrue(matcher.find());
        assertEquals(5, matcher.start());
        
        Pattern pattern = Pattern.compile(regex);
        final Matcher matcher2 = pattern.matcher(input);
        assertFalse(matcher2.find());
        
        Pattern.compile(regex, Pattern.CANON_EQ);
    }
    
    @Test
    public void replaceAll() {
        final String regex = "a*b";
        final String input = "aabfooaabfooabfoob";
        
        final String[] values = java.util.regex.Pattern.compile(regex).split(input);
        java.util.regex.Matcher matcher = java.util.regex.Pattern.compile(regex).matcher(input);
        assertEquals("-foo-foo-foo-", matcher.replaceAll("-"));
        
        Pattern pattern = Pattern.compile(regex);
        final String[] values2 = pattern.split(input);
        final Matcher matcher2 = pattern.matcher(input);
        assertEquals("-foo-foo-foo-", matcher2.replaceAll("-"));
    }
    
    @Test
    public void replaceAll2() {
        final String regex = "(\n|\r\n|\r|\u0085|\u2028|\u2029)";
        final String input = "あいうえお\nかきくけこ\r\nさしすせそ\rたちつてと\u0085なにぬねの\u2028はひふへほ\u2029まみむめも";
//        final String input = "aaa\r\nbbb";
        final String replacement = "<br/>\\n\\$0";
        
        java.util.regex.Matcher matcher = java.util.regex.Pattern.compile(regex).matcher(input);
        final String replaced = matcher.replaceAll(replacement);
        System.out.println("replaceAll2:replaced :[" + replaced + "]\n");
        
        Pattern pattern = Pattern.compile(regex);
        final Matcher matcher2 = pattern.matcher(input);
        final String replaced2 = matcher2.replaceAll(replacement);
        System.out.println("replaceAll2:replaced2:[" + replaced2 + "]\n");
        
        assertEquals(replaced, replaced2);
    }
    
    @Test
    public void replaceAll3() {
        final String regex = "(\n|\r\n|\r|\u0085|\u2028|\u2029)";
        final String input = "あいうえお\nかきくけこ\r\nさしすせそ\rたちつてと\u0085なにぬねの\u2028はひふへほ\u2029まみむめも";
        final String replacement = "<br/>\\n\\\\$0";
        
        java.util.regex.Matcher matcher = java.util.regex.Pattern.compile(regex).matcher(input);
        final String replaced = matcher.replaceAll(replacement);
        System.out.println("replaceAll3:replaced :[" + replaced + "]\n");
        
        Pattern pattern = Pattern.compile(regex);
        final Matcher matcher2 = pattern.matcher(input);
        final String replaced2 = matcher2.replaceAll(replacement);
        System.out.println("replaceAll3:replaced2:[" + replaced2 + "]\n");
        
        assertEquals(replaced, replaced2);
    }
    
    @Test
    public void replaceAll4() {
        final String regex = "(\n|\r\n|\r|\u0085|\u2028|\u2029)";
        final String input = "あいうえお\nかきくけこ\r\nさしすせそ\rたちつてと\u0085なにぬねの\u2028はひふへほ\u2029まみむめも";
        final String replacement = "<br/>\\\\n\\\\$0";
        
        java.util.regex.Matcher matcher = java.util.regex.Pattern.compile(regex).matcher(input);
        final String replaced = matcher.replaceAll(replacement);
        System.out.println("replaceAll4:replaced :[" + replaced + "]\n");
        
        Pattern pattern = Pattern.compile(regex);
        final Matcher matcher2 = pattern.matcher(input);
        final String replaced2 = matcher2.replaceAll(replacement);
        System.out.println("replaceAll4:replaced2:[" + replaced2 + "]\n");
        
        assertEquals(replaced, replaced2);
    }
    
    @Test
    public void replaceAll5() {
        final String regex = "(\n|\r\n|\r|\u0085|\u2028|\u2029)";
        final String input = "あいうえお\nかきくけこ\r\nさしすせそ\rたちつてと\u0085なにぬねの\u2028はひふへほ\u2029まみむめも";
        final String replacement = "<br/>\\n";
        
        java.util.regex.Matcher matcher = java.util.regex.Pattern.compile(regex).matcher(input);
        final String replaced = matcher.replaceAll(replacement);
        System.out.println("replaceAll5:replaced :[" + replaced + "]\n");
        
        Pattern pattern = Pattern.compile(regex);
        final Matcher matcher2 = pattern.matcher(input);
        final String replaced2 = matcher2.replaceAll(replacement);
        System.out.println("replaceAll5:replaced2:[" + replaced2 + "]\n");
        
        assertEquals(replaced, replaced2);
    }
    
    @Test
    public void replaceFirst() {
        final String regex = "dog";
        final String input = "zzzdogzzzdogzzz";
        
        java.util.regex.Matcher matcher = java.util.regex.Pattern.compile(regex).matcher(input);
        assertEquals("zzzcatzzzdogzzz", matcher.replaceFirst("cat"));
        
        Pattern pattern = Pattern.compile(regex);
        final Matcher matcher2 = pattern.matcher(input);
        assertEquals("zzzcatzzzdogzzz", matcher2.replaceFirst("cat"));        
    }
    
    @Test
    public void replaceFirst2() {
        final String regex = "dog";
        final String input = "zzzdogzzzdogzzz";
        final String replacement = "cat_and_$0";
        
        java.util.regex.Matcher matcher = java.util.regex.Pattern.compile(regex).matcher(input);
        final String replaced = matcher.replaceFirst(replacement);
        assertEquals("zzzcat_and_dogzzzdogzzz", replaced);
        
        Pattern pattern = Pattern.compile(regex);
        final Matcher matcher2 = pattern.matcher(input);
        assertEquals(replaced, matcher2.replaceFirst(replacement));        
    }
    
    @Test
    public void quoteReplacement() {
        final String replacement = "cat_\\and_\\$0";
        
        final String quoted = java.util.regex.Matcher.quoteReplacement(replacement);
        System.out.println("quoteReplacement:quoted :[" + quoted + "]\n");
//        assertEquals("zzzcat_and_dogzzzdogzzz", replaced);
        
        assertEquals(quoted, Matcher.quoteReplacement(replacement));        
    }
    
    @Test
    public void useTransparentBounds() {
        final String regex = "123(?=45)";
        final String input = "012345";
        
        java.util.regex.Matcher matcher = java.util.regex.Pattern.compile(regex).matcher(input);
        if (!matcher.hasTransparentBounds()) {
            matcher.useTransparentBounds(true);
        }
        assertTrue(matcher.find());
        assertEquals("123", matcher.group());

        matcher.region(1, 4);
        assertTrue(matcher.find());
        assertEquals("123", matcher.group());
        
        matcher.useTransparentBounds(false);
        matcher.region(1, 4);        
        assertFalse(matcher.find());
        
        final Matcher matcher2 = Pattern.compile(regex).matcher(input);
        assertTrue(matcher2.find());
        assertEquals("123", matcher2.group());
 
        matcher2.region(1, 4);
        assertTrue(matcher2.find());
        assertEquals("123", matcher2.group());
    }
}